/**
 * jQuery cachingSlider plugin
 * @name jquery.cachingSlider.js
 * @author Marco Nitsche - http://www.codii.de
 * @version 1.9
 * @date October 10, 2012
 * @category jQuery plugin
 * @copyright (c) 2012 Marco Nitsche, codii
 * @license Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 * @example http://www.codii.de/demo/cachingSlider/
 */

var Methods = {

	cache: function (index) {
		'use strict';
		index = parseInt(index, 10);

		var $this = this,
				Data = $this.data('cachingSlider');

		if (index < Data.elements.length && typeof Data.elements[index] !== 'object') {
			var Ele = jQuery(Data.elements[index]);

			if (Data.slideTo === 'left' || Data.slideTo === 'right') {
				Ele.css('display', 'inline-block');
			} else if (Data.slideTo === 'top' || Data.slideTo === 'bottom') {
				Ele.css('display', 'block');
			} else {
				Ele.css({'left': 0, 'position': 'absolute', 'top': 0});
			}

			if (Data.slideTo === 'fade') {
				$this.prepend(Ele);
				$this.height(Math.max(Ele.outerHeight(true), $this.height()));
			} else {
				$this.append(Ele);
			}

			Data.elements[index] = Ele;
			Data.cached.call($this, Ele);
			Data._cacheCount++;
			if (Data.elements.length === Data._cacheCount) Data.complete.call($this);
		}

	},

	init: function (options) {
		'use strict';
		var Settings = {
			'auto': true, // Autostarts the slider, after initialization
			'cacheCount': 2, // The amount of objects to be Cached
			'cached': function (obj) {}, // A function called after caching each object, 'this' refers to the Container
			'complete': function () {}, // A function called after caching all objects, 'this' refers to the Container
			'displayCount': 1, // The amount of objects to be displayed
			'duration': 800, // A string or number determining the duration of the slide
			'easing': 'swing', // The Type of the animation. Possible values are 'swing', 'linear'. Will be extended by jQueryUI
			'elements': [], // An Array of objects to be slided
			'pause': 2800, // The time in miliseconds, between the slides. Use 1 for instant sliding
			'slide': function (obj, index) {}, // A function called before sliding each object, 'this' refers to the Container, index is the index of the upcoming object
			'slideTo': 'left' // The direction, in wich will be slided or 'fade' for fading. Possible values are 'left', 'right', 'top', 'bottom', 'fade'
		};

		if (options) jQuery.extend(Settings, options);

		return this.each(function () {
			var $this = jQuery(this),
					Data = jQuery.extend(true, {'animating': false, '_cacheCount': 0, 'index': 0, 'interval': null}, Settings);

			$this.data('cachingSlider', Data);
			Data.animating = true;
			if (Data.elements.length < (Data.cacheCount + Data.displayCount)) Data.cacheCount = Data.elements.length - Data.displayCount;

			if (Data.slideTo === 'left' || Data.slideTo === 'right') {
				$this.css('white-space', 'nowrap');
			} else {
				$this.css('white-space', 'normal');
			}

			// Die ersten n Objekte einfügen
			for (var i = 0; i < Data.cacheCount + Data.displayCount && i < Data.elements.length; i++) {
				var Ele = jQuery(Data.elements[i]);
				Methods.cache.call($this, i);
			}

			if (Data.cacheCount > 0) { // leave the Slider deactivated, if there are too less elements
				Data.animating = false;

				if (Data.auto) {

					Data.interval = setInterval(function () {
						Methods.next.call($this);
					}, Data.pause);

				}

			}

		});

	},

	next: function () {
		Methods.slideToOffset.call(this, 1);

		return this.each(function () {
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			for (var i = 0; Data.elements.length != Data._cacheCount && i < Data.cacheCount; i++) {

				if (Data.index + Data.displayCount - 1 + Data.cacheCount - 1 + i + 1 < Data.elements.length) {
					Methods.cache.call($this, Data.index + Data.displayCount - 1 + Data.cacheCount - 1 + i + 1);
				} else {
					Methods.cache.call($this, (Data.elements.length - Data.displayCount - Data.cacheCount - Data.index + 1 - i) * -1);
				}

			}

		});

	},

	previous: function () {
		Methods.slideToOffset.call(this, -1);

		return this.each(function () {
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			for (var i = 0; Data.elements.length != Data._cacheCount && i < Data.cacheCount; i++) {

				if (Data.index - i > 0) {
					Methods.cache.call($this, Data.index - i - 1);
				} else {
					Methods.cache.call($this, Data.elements.length - 1 - i - 1 + Data.index);
				}

			}

		});

	},

	slideToIndex: function (index) {
		index = parseInt(index, 10);

		return this.each(function () {
			'use strict';
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			if (!Data.animating) {
				Data.animating = true;
				if (index >= Data.elements.length) index = index - Data.elements.length;
				Methods.cache.call($this, index);
				var $Current = Data.elements[Data.index],
						$Next,
						$LastShown,
						Ini,
						Props = {},
						Reverse = 0;

				switch (Data.slideTo) {
					case 'bottom':
					case 'right':
						Reverse = 1;
						if ((index > Data.index && !(index == (Data.elements.length - 1) && Data.index == 0)) || (Data.index == (Data.elements.length - 1) && index == 0)) Reverse = 2;
						break;
				}

				if (Reverse == 1 || (Reverse != 2 && ((index > Data.index && !(index == (Data.elements.length - 1) && Data.index == 0)) || (Data.index == (Data.elements.length - 1) && index == 0)))) {
					Ini = {};

					if (index + Data.displayCount <= Data.elements.length) {
						$Next = Data.elements[index + Data.displayCount - 1];
					} else {
						$Next = Data.elements[(Data.elements.length - Data.displayCount - index + 1) * -1];
					}

					if (Data.index + Data.displayCount <= Data.elements.length) {
						$LastShown = Data.elements[Data.index + Data.displayCount - 1];
					} else {
						$LastShown = Data.elements[(Data.elements.length - Data.displayCount - Data.index + 1) * -1];
					}

					if (Data.slideTo == 'left' || Data.slideTo == 'right') {
						Ini = {'margin-left': $Current.css('margin-left')};
						if (isNaN(Ini['margin-left'])) Ini['margin-left'] = parseFloat(Ini['margin-left']) || 0;
						Props = {'margin-left': ('-=' + $Current.outerWidth(true))};
					} else if (Data.slideTo == 'top' || Data.slideTo == 'bottom') {
						Ini = {'margin-top': $Current.css('margin-top')};
						if (isNaN(Ini['margin-top'])) Ini['margin-top'] = parseFloat(Ini['margin-top']) || 0;
						Props = {'margin-top': ('-=' + (parseFloat($Current.outerHeight(true)) - parseFloat(Ini['margin-top'])))};
					} else {
						Ini = {'opacity': $Current.css('opacity')};
						if (isNaN(Ini['opacity'])) Ini['opacity'] = parseFloat(Ini['opacity']) || 0;
						Props = {'opacity': ('-=' + Ini.opacity)};
					}

					switch (Data.slideTo) {
						case 'bottom':
						case 'right':
							$LastShown.after($Next);
							Data.slide.call($this, $Current, index);
							break;

						case 'fade':
							$Current.before($Next);
							Data.slide.call($this, $Current, index);
							break;

						default:
							$LastShown.after($Next);
							Data.slide.call($this, $Current, index);
					}

					$Current.animate(Props, {
						complete: function () {

							if (Data.slideTo == 'fade') {
								$this.prepend($Current);
								$this.height(Math.max($Next.outerHeight(true), $this.height()));
							} else {
								$this.append($Current);
							}

							$Current.css(Ini);
							Data.index = index;
							Data.animating = false;
						},
						duration:Data.duration,
						easing:Data.easing,
						queue:false
					});

				} else {
					Ini = 0;
					$Next = Data.elements[index];
					$LastShown = $Current;

					if (Data.slideTo == 'left' || Data.slideTo == 'right') {
						Ini = $Next.css('margin-left');
						if (isNaN(Ini)) Ini = parseFloat(Ini) || 0;
						$Next.css('margin-left', (parseFloat($Next.outerWidth(true)) - parseFloat(Ini)) * -1);
						Props = {'margin-left': Ini};
					} else if (Data.slideTo == 'top' || Data.slideTo == 'bottom') {
						Ini = $Next.css('margin-top');
						if (isNaN(Ini)) Ini = parseFloat(Ini) || 0;
						$Next.css('margin-top', (parseFloat($Next.outerHeight(true)) - parseFloat(Ini) * 2) * -1);
						Props = {'margin-top': Ini};
					} else {
						Ini = $Next.css('opacity');
						if (isNaN(Ini)) Ini = parseFloat(Ini) || 0;
						$Next.css('opacity', 0);
						Props = {'opacity': ('+=' + Ini)};
					}

					switch (Data.slideTo) {
						case 'bottom':
						case 'right':
							$LastShown.before($Next);
							Data.slide.call($this, $Next, index);
							break;

						case 'fade':
							$Current.after($Next);
							Data.slide.call($this, $Next, index);
							break;

						default:
							$LastShown.before($Next);
							Data.slide.call($this, $Next, index);
					}

					$Next.animate(Props, {

						complete: function () {
							Data.index = index;
							Data.animating = false;
						},

						duration:Data.duration,
						easing:Data.easing,
						queue:false
					});

				}

			}

		});

	},

	slideToOffset: function (offset) {
		offset = parseInt(offset, 10);

		return this.each(function () {
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			if (offset > 0) {

				if (Data.index < (Data.elements.length - offset)) {
					Methods.slideToIndex.call($this, (Data.index + offset));
				} else {
					Methods.slideToIndex.call($this, 0);
				}

			} else {

				if (Data.index > (-1 - offset)) {
					Methods.slideToIndex.call($this, (Data.index - 1));
				} else {
					Methods.slideToIndex.call($this, (Data.elements.length - 1));
				}

			}

		});

	},

	start: function () {

		return this.each(function () {
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			clearInterval(Data.interval);

			Data.interval = setInterval(function () {
				Methods.next.call($this);
			}, Data.pause);

		});

	},

	stop: function () {

		return this.each(function () {
			var $this = jQuery(this),
					Data = $this.data('cachingSlider');

			clearInterval(Data.interval);
		});

	}

};

jQuery.fn.cachingSlider = function (method) {

	// Method calling logic
	if (Methods[method]) {
		return Methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
	} else if (typeof method === 'object' || !method) {
		return Methods.init.apply(this, arguments);
	} else {
		jQuery.error('Method ' +  method + ' does not exist on jQuery.cachingSlider');
		return this;
	}

};